@@ -2,7 +2,7 @@ package MyModel4::book;
 
 use base 'MyModel4';
 
-sub table { 'company' }
+sub table { 'table1' }
 
 sub list { shift->select }
 
